@@ -1,11 +1,6 @@
-from collections import namedtuple
-from .borg_thread import BorgThread
+from .borg_thread import BorgThread, FakeProfile, FakeRepo
+from vorta.i18n import trans_late
 from vorta.models import RepoModel
-from vorta.utils import keyring
-import os
-
-FakeRepo = namedtuple('Repo', ['url', 'id', 'extra_borg_arguments'])
-FakeProfile = namedtuple('FakeProfile', ['repo', 'name', 'ssh_key'])
 
 
 class BorgInfoThread(BorgThread):
@@ -19,9 +14,9 @@ class BorgInfoThread(BorgThread):
         Used to validate existing repository when added.
         """
 
-        # Build fake profile because we don't have it in the DB yet.
+        # Build fake profile because we don't have it in the DB yet. Assume unencrypted.
         profile = FakeProfile(
-            FakeRepo(params['repo_url'], 999, params['extra_borg_arguments']),
+            FakeRepo(params['repo_url'], 999, params['extra_borg_arguments'], 'none'),
             'New Repo',
             params['ssh_key']
         )
@@ -35,12 +30,18 @@ class BorgInfoThread(BorgThread):
         cmd = ["borg", "info", "--info", "--json", "--log-json"]
         cmd.append(profile.repo.url)
 
-        os.environ['BORG_UNKNOWN_UNENCRYPTED_REPO_ACCESS_IS_OK'] = "yes"
+        ret['additional_env'] = {
+            'BORG_UNKNOWN_UNENCRYPTED_REPO_ACCESS_IS_OK': "yes",
+            'BORG_RSH': 'ssh -oStrictHostKeyChecking=no'
+        }
+
+        ret['password'] = params['password']  # Empty password is '', which disables prompt
+        if params['password'] != '':
+            # Cannot tell if repo has encryption, assuming based off of password
+            if not cls.keyring.is_unlocked:
+                ret['message'] = trans_late('messages', 'Please unlock your password manager.')
+                return ret
 
-        if params['password'] == '':
-            ret['password'] = '999999'  # Dummy password if the user didn't supply one. To avoid prompt.
-        else:
-            ret['password'] = params['password']
         ret['ok'] = True
         ret['cmd'] = cmd
 
@@ -60,7 +61,7 @@ class BorgInfoThread(BorgThread):
             if 'encryption' in result['data']:
                 new_repo.encryption = result['data']['encryption']['mode']
             if new_repo.encryption != 'none':
-                keyring.set_password("vorta-repo", new_repo.url, result['params']['password'])
+                self.keyring.set_password("vorta-repo", new_repo.url, result['params']['password'])
 
             new_repo.extra_borg_arguments = result['params']['extra_borg_arguments']
 
