@@ -3,16 +3,13 @@ from .borg_thread import BorgThread
 
 class BorgListArchiveThread(BorgThread):
 
-    def log_event(self, msg):
-        self.app.backup_log_event.emit(msg)
-
     def started_event(self):
         self.app.backup_started_event.emit()
-        self.app.backup_log_event.emit(self.tr('Getting archive content...'))
+        self.app.backup_progress_event.emit(self.tr('Getting archive content...'))
 
     def finished_event(self, result):
         self.app.backup_finished_event.emit(result)
-        self.app.backup_log_event.emit(self.tr('Done getting archive content.'))
+        self.app.backup_progress_event.emit(self.tr('Done getting archive content.'))
         self.result.emit(result)
 
     @classmethod
