@@ -16,7 +16,7 @@ from playhouse.migrate import SqliteMigrator, migrate
 from vorta.i18n import trans_late
 from vorta.utils import slugify
 
-SCHEMA_VERSION = 15
+SCHEMA_VERSION = 17
 
 db = pw.Proxy()
 
@@ -46,6 +46,7 @@ class RepoModel(pw.Model):
     unique_csize = pw.IntegerField(null=True)
     total_size = pw.IntegerField(null=True)
     total_unique_chunks = pw.IntegerField(null=True)
+    create_backup_cmd = pw.CharField(default='')
     extra_borg_arguments = pw.CharField(default='')
 
     def is_remote_repo(self):
@@ -87,8 +88,8 @@ class BackupProfileModel(pw.Model):
     prune_month = pw.IntegerField(default=6)
     prune_year = pw.IntegerField(default=2)
     prune_keep_within = pw.CharField(default='10H', null=True)
-    new_archive_name = pw.CharField(default="{hostname}-{profile_slug}-{now:%Y-%m-%dT%H:%M:%S}")
-    prune_prefix = pw.CharField(default="{hostname}-{profile_slug}-")
+    new_archive_name = pw.CharField(default="{hostname}-{now:%Y-%m-%d-%H%M%S}")
+    prune_prefix = pw.CharField(default="{hostname}-")
     pre_backup_cmd = pw.CharField(default='')
     post_backup_cmd = pw.CharField(default='')
     dont_run_on_metered_networks = pw.BooleanField(default=True)
@@ -106,6 +107,9 @@ class BackupProfileModel(pw.Model):
 class SourceFileModel(pw.Model):
     """A folder to be backed up, related to a Backup Configuration."""
     dir = pw.CharField()
+    dir_size = pw.BigIntegerField(default=-1)
+    dir_files_count = pw.BigIntegerField(default=-1)
+    path_isdir = pw.BooleanField(default=False)
     profile = pw.ForeignKeyField(BackupProfileModel, default=1)
     added_at = pw.DateTimeField(default=datetime.utcnow)
 
@@ -193,6 +197,7 @@ def _apply_schema_update(current_schema, version_after, *operations):
 
 
 def get_misc_settings():
+    ''' Global settings that apply per platform '''
     # Default settings for all platforms.
     settings = [
         {
@@ -215,6 +220,16 @@ def get_misc_settings():
             'label': trans_late('settings',
                                 'Open main window on startup')
         },
+        {
+            'key': 'get_srcpath_datasize', 'value': True, 'type': 'checkbox',
+            'label': trans_late('settings',
+                                'Get statistics of file/folder when added')
+        },
+        {
+            'key': 'override_mount_permissions', 'value': False, 'type': 'checkbox',
+            'label': trans_late('settings',
+                                'Try to replace existing permissions when mounting an archive.')
+        },
         {
             'key': 'previous_profile_id', 'str_value': '1', 'type': 'internal',
             'label': 'Previously selected profile'
@@ -241,9 +256,26 @@ def get_misc_settings():
                                     'Include pre-release versions when checking for updates')
             },
         ]
+    else:
+        settings += [
+            {
+                'key': 'enable_background_question', 'value': True, 'type': 'checkbox',
+                'label': trans_late('settings',
+                                    'Display background exit dialog')
+            },
+            {
+                'key': 'disable_background_state', 'value': False, 'type': 'internal',
+                'label': 'Previous background exit button state'
+            }
+        ]
     return settings
 
 
+def cleanup_db():
+    # Clean up database
+    db.execute_sql("VACUUM")
+
+
 def init_db(con=None):
     if con is not None:
         os.umask(0o0077)
@@ -345,7 +377,7 @@ def init_db(con=None):
             with db.atomic():
                 size = 1000
                 for i in range(0, len(data), size):
-                    ArchiveModel.insert_many(data[i:i + size], fields=fields).execute()
+                    ArchiveModel.insert_many(data[i: i + size], fields=fields).execute()
 
         _apply_schema_update(current_schema, 13)
 
@@ -362,6 +394,24 @@ def init_db(con=None):
                                 'dont_run_on_metered_networks', pw.BooleanField(default=True))
         )
 
+    if current_schema.version < 16:
+        _apply_schema_update(
+            current_schema, 16,
+            migrator.add_column(SourceFileModel._meta.table_name,
+                                'dir_size', pw.BigIntegerField(default=-1)),
+            migrator.add_column(SourceFileModel._meta.table_name,
+                                'dir_files_count', pw.BigIntegerField(default=-1)),
+            migrator.add_column(SourceFileModel._meta.table_name,
+                                'path_isdir', pw.BooleanField(default=False))
+        )
+
+    if current_schema.version < 17:
+        _apply_schema_update(
+            current_schema, 17,
+            migrator.add_column(RepoModel._meta.table_name,
+                                'create_backup_cmd', pw.CharField(default=''))
+        )
+
     # Create missing settings and update labels. Leave setting values untouched.
     for setting in get_misc_settings():
         s, created = SettingsModel.get_or_create(key=setting['key'], defaults=setting)
@@ -370,4 +420,4 @@ def init_db(con=None):
 
     # Delete old log entries after 3 months.
     three_months_ago = datetime.now() - rd(months=3)
-    EventLogModel.delete().where(EventLogModel.start_time < three_months_ago)
+    EventLogModel.delete().where(EventLogModel.start_time < three_months_ago).execute()
