@@ -5,7 +5,8 @@ from datetime import timedelta
 from PyQt5 import QtCore, uic
 from PyQt5.QtGui import QDesktopServices
 from PyQt5.QtWidgets import (QHeaderView, QMessageBox, QTableView,
-                             QTableWidgetItem)
+                             QTableWidgetItem, QInputDialog, QMenu,
+                             QToolButton)
 
 from vorta.borg.check import BorgCheckThread
 from vorta.borg.delete import BorgDeleteThread
@@ -16,10 +17,12 @@ from vorta.borg.list_repo import BorgListRepoThread
 from vorta.borg.mount import BorgMountThread
 from vorta.borg.prune import BorgPruneThread
 from vorta.borg.umount import BorgUmountThread
+from vorta.borg.rename import BorgRenameThread
 from vorta.i18n import trans_late
 from vorta.models import ArchiveModel, BackupProfileMixin
 from vorta.utils import (choose_file_dialog, format_archive_name, get_asset,
                          get_mount_points, pretty_bytes)
+from vorta.views.source_tab import SizeItem
 from vorta.views.diff_dialog import DiffDialog
 from vorta.views.diff_result import DiffResult
 from vorta.views.extract_dialog import ExtractDialog
@@ -32,11 +35,12 @@ ArchiveTabUI, ArchiveTabBase = uic.loadUiType(uifile)
 class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     prune_intervals = ['hour', 'day', 'week', 'month', 'year']
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, app=None):
         super().__init__(parent)
         self.setupUi(parent)
         self.mount_points = {}
         self.menu = None
+        self.app = app
         self.toolBox.setCurrentIndex(0)
 
         header = self.archiveTable.horizontalHeader()
@@ -58,16 +62,17 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         self.archiveTable.setTextElideMode(QtCore.Qt.ElideLeft)
         self.archiveTable.setAlternatingRowColors(True)
         self.archiveTable.cellDoubleClicked.connect(self.cell_double_clicked)
-        self.archiveTable.itemSelectionChanged.connect(self.update_mount_button_text)
         self.archiveTable.setSortingEnabled(True)
 
-        self.mountButton.clicked.connect(self.mount_action)
         self.listButton.clicked.connect(self.list_action)
         self.pruneButton.clicked.connect(self.prune_action)
         self.checkButton.clicked.connect(self.check_action)
-        self.extractButton.clicked.connect(self.list_archive_action)
         self.diffButton.clicked.connect(self.diff_action)
-        self.deleteButton.clicked.connect(self.delete_action)
+
+        self.archiveActionMenu = QMenu(parent=self)
+        self.archiveActionMenu.aboutToShow.connect(self.showArchiveActionMenu)
+        self.archiveActionButton.setMenu(self.archiveActionMenu)
+        self.archiveActionButton.setPopupMode(QToolButton.InstantPopup)
 
         self.archiveNameTemplate.textChanged.connect(
             lambda tpl, key='new_archive_name': self.save_archive_template(tpl, key))
@@ -75,20 +80,22 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             lambda tpl, key='prune_prefix': self.save_archive_template(tpl, key))
 
         self.populate_from_profile()
+        self.selected_archives = None
         self.set_icons()
 
-        self.selected_archives = None
-
     def set_icons(self):
-        self.extractButton.setIcon(get_colored_icon('cloud-download'))
-        self.mountButton.setIcon(get_colored_icon('folder-open'))
+        "Used when changing between light- and dark mode"
         self.checkButton.setIcon(get_colored_icon('check-circle'))
-        self.deleteButton.setIcon(get_colored_icon('trash'))
         self.diffButton.setIcon(get_colored_icon('stream-solid'))
         self.pruneButton.setIcon(get_colored_icon('cut'))
         self.listButton.setIcon(get_colored_icon('refresh'))
         self.toolBox.setItemIcon(0, get_colored_icon('tasks'))
         self.toolBox.setItemIcon(1, get_colored_icon('cut'))
+        self.archiveActionButton.setIcon(get_colored_icon('ellipsis-v'))
+
+    def cancel_action(self):
+        self._set_status(self.tr("Action cancelled."))
+        self._toggle_all_buttons(True)
 
     def _set_status(self, text):
         self.mountErrors.setText(text)
@@ -96,11 +103,36 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
     def _toggle_all_buttons(self, enabled=True):
         for button in [self.checkButton, self.listButton, self.pruneButton,
-                       self.mountButton, self.extractButton, self.deleteButton,
-                       self.diffButton]:
+                       self.diffButton, self.archiveActionButton]:
             button.setEnabled(enabled)
             button.repaint()
 
+    def showArchiveActionMenu(self):
+        archive_name = self.selected_archive_name()
+        menu = self.archiveActionMenu
+        menu.clear()
+
+        if not archive_name:
+            action = menu.addAction(self.tr("Select an archive first."))
+            action.setEnabled(False)
+            return menu
+
+        extractAction = menu.addAction("Extract", self.list_archive_action)
+        deleteAction = menu.addAction("Delete", self.delete_action)
+        renameAction = menu.addAction("Rename", self.rename_action)
+
+        if archive_name in self.mount_points:
+            unmountAction = menu.addAction("Unmount", self.umount_action)
+            unmountAction.setIcon(get_colored_icon('eject'))
+        else:
+            mountAction = menu.addAction("Mount", self.mount_action)
+            mountAction.setIcon(get_colored_icon('folder-open'))
+
+        extractAction.setIcon(get_colored_icon('cloud-download'))
+        deleteAction.setIcon(get_colored_icon('trash'))
+        renameAction.setIcon(get_colored_icon('edit'))
+        return menu
+
     def populate_from_profile(self):
         """Populate archive list and prune settings from profile."""
         profile = self.profile()
@@ -109,12 +141,14 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             self.toolBox.setItemText(0, self.tr('Archives for %s') % profile.repo.url)
             archives = [s for s in profile.repo.archives.select().order_by(ArchiveModel.time.desc())]
 
+            sorting = self.archiveTable.isSortingEnabled()
+            self.archiveTable.setSortingEnabled(False)
             for row, archive in enumerate(archives):
                 self.archiveTable.insertRow(row)
 
                 formatted_time = archive.time.strftime('%Y-%m-%d %H:%M')
                 self.archiveTable.setItem(row, 0, QTableWidgetItem(formatted_time))
-                self.archiveTable.setItem(row, 1, QTableWidgetItem(pretty_bytes(archive.size)))
+                self.archiveTable.setItem(row, 1, SizeItem(pretty_bytes(archive.size)))
                 if archive.duration is not None:
                     formatted_duration = str(timedelta(seconds=round(archive.duration)))
                 else:
@@ -130,6 +164,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 self.archiveTable.setItem(row, 4, QTableWidgetItem(archive.name))
 
             self.archiveTable.setRowCount(len(archives))
+            self.archiveTable.setSortingEnabled(sorting)
             item = self.archiveTable.item(0, 0)
             self.archiveTable.scrollToItem(item)
             self._toggle_all_buttons(enabled=True)
@@ -178,7 +213,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 archive_name = archive_cell.text()
                 params['cmd'][-1] += f'::{archive_name}'
 
-        thread = BorgCheckThread(params['cmd'], params, parent=self)
+        thread = BorgCheckThread(params['cmd'], params, parent=self.app)
         thread.updated.connect(self._set_status)
         thread.result.connect(self.check_result)
         self._toggle_all_buttons(False)
@@ -191,11 +226,13 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     def prune_action(self):
         params = BorgPruneThread.prepare(self.profile())
         if params['ok']:
-            thread = BorgPruneThread(params['cmd'], params, parent=self)
+            thread = BorgPruneThread(params['cmd'], params, parent=self.app)
             thread.updated.connect(self._set_status)
             thread.result.connect(self.prune_result)
             self._toggle_all_buttons(False)
             thread.start()
+        else:
+            self._set_status(params['message'])
 
     def prune_result(self, result):
         if result['returncode'] == 0:
@@ -207,11 +244,13 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     def list_action(self):
         params = BorgListRepoThread.prepare(self.profile())
         if params['ok']:
-            thread = BorgListRepoThread(params['cmd'], params, parent=self)
+            thread = BorgListRepoThread(params['cmd'], params, parent=self.app)
             thread.updated.connect(self._set_status)
             thread.result.connect(self.list_result)
             self._toggle_all_buttons(False)
             thread.start()
+        else:
+            self._set_status(params['message'])
 
     def list_result(self, result):
         self._toggle_all_buttons(True)
@@ -227,12 +266,6 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 return archive_cell.text()
         return None
 
-    def set_mount_button_mode(self, mode):
-        self.mountButton.clicked.disconnect()
-        mount = (mode == 'Mount')
-        self.mountButton.setText(self.tr('Mount') if mount else self.tr('Unmount'))
-        self.mountButton.clicked.connect(self.mount_action if mount else self.umount_action)
-
     def mount_action(self):
         profile = self.profile()
         params = BorgMountThread.prepare(profile)
@@ -254,7 +287,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                     self.mount_points[params['current_archive']] = mount_point[0]
                 if params['ok']:
                     self._toggle_all_buttons(False)
-                    thread = BorgMountThread(params['cmd'], params, parent=self)
+                    thread = BorgMountThread(params['cmd'], params, parent=self.app)
                     thread.updated.connect(self.mountErrors.setText)
                     thread.result.connect(self.mount_result)
                     thread.start()
@@ -266,7 +299,6 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
         self._toggle_all_buttons(True)
         if result['returncode'] == 0:
             self._set_status(self.tr('Mounted successfully.'))
-            self.update_mount_button_text()
             if result['params'].get('current_archive'):
                 archive_name = result['params']['current_archive']
                 row = self.row_of_archive(archive_name)
@@ -275,7 +307,6 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
     def umount_action(self):
         archive_name = self.selected_archive_name()
-
         mount_point = self.mount_points.get(archive_name)
 
         if mount_point is not None:
@@ -289,7 +320,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
             if os.path.normpath(mount_point) in params['active_mount_points']:
                 params['cmd'].append(mount_point)
-                thread = BorgUmountThread(params['cmd'], params, parent=self)
+                thread = BorgUmountThread(params['cmd'], params, parent=self.app)
                 thread.updated.connect(self.mountErrors.setText)
                 thread.result.connect(self.umount_result)
                 thread.start()
@@ -299,14 +330,16 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
     def umount_result(self, result):
         self._toggle_all_buttons(True)
+        archive_name = result['params']['current_archive']
         if result['returncode'] == 0:
             self._set_status(self.tr('Un-mounted successfully.'))
-            archive_name = result['params']['current_archive']
             del self.mount_points[archive_name]
-            self.update_mount_button_text()
             row = self.row_of_archive(archive_name)
             item = QTableWidgetItem('')
             self.archiveTable.setItem(row, 3, item)
+        else:
+            self._set_status(self.tr('Unmounting failed. Make sure no programs are using {}').format(
+                self.mount_points.get(archive_name)))
 
     def save_prune_setting(self, new_value=None):
         profile = self.profile()
@@ -333,7 +366,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 self._set_status('')
                 self._toggle_all_buttons(False)
 
-                thread = BorgListArchiveThread(params['cmd'], params, parent=self)
+                thread = BorgListArchiveThread(params['cmd'], params, parent=self.app)
                 thread.updated.connect(self.mountErrors.setText)
                 thread.result.connect(self.list_archive_result)
                 thread.start()
@@ -343,14 +376,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     def list_archive_result(self, result):
         self._set_status('')
         if result['returncode'] == 0:
-            archive = ArchiveModel.get(name=result['params']['archive_name'])
-            window = ExtractDialog(result['data'], archive)
-            self._toggle_all_buttons(True)
-            window.setParent(self, QtCore.Qt.Sheet)
-            self._window = window  # for testing
-            window.show()
-
-            if window.exec_():
+            def process_result():
                 def receive():
                     extraction_folder = dialog.selectedFiles()
                     if extraction_folder:
@@ -358,7 +384,7 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                             self.profile(), archive.name, window.selected, extraction_folder[0])
                         if params['ok']:
                             self._toggle_all_buttons(False)
-                            thread = BorgExtractThread(params['cmd'], params, parent=self)
+                            thread = BorgExtractThread(params['cmd'], params, parent=self.app)
                             thread.updated.connect(self.mountErrors.setText)
                             thread.result.connect(self.extract_archive_result)
                             thread.start()
@@ -368,17 +394,17 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
                 dialog = choose_file_dialog(self, self.tr("Choose Extraction Point"), want_folder=True)
                 dialog.open(receive)
 
+            archive = ArchiveModel.get(name=result['params']['archive_name'])
+            window = ExtractDialog(result['data'], archive)
+            self._toggle_all_buttons(True)
+            window.setParent(self, QtCore.Qt.Sheet)
+            self._window = window  # for testing
+            window.show()
+            window.accepted.connect(process_result)
+
     def extract_archive_result(self, result):
         self._toggle_all_buttons(True)
 
-    def update_mount_button_text(self):
-        archive_name = self.selected_archive_name()
-        if not archive_name:
-            return
-
-        mode = 'Unmount' if archive_name in self.mount_points else 'Mount'
-        self.set_mount_button_mode(mode)
-
     def cell_double_clicked(self, row, column):
         if column == 3:
             archive_name = self.selected_archive_name()
@@ -411,14 +437,14 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             self._set_status(params['message'])
             return
 
-        archive_name = self.selected_archive_name()
-        if archive_name is not None:
+        self.archive_name = self.selected_archive_name()
+        if self.archive_name is not None:
             if not self.confirm_dialog(trans_late('ArchiveTab', "Confirm deletion"),
                                        trans_late('ArchiveTab', "Are you sure you want to delete the archive?")):
                 return
-            params['cmd'][-1] += f'::{archive_name}'
+            params['cmd'][-1] += f'::{self.archive_name}'
 
-            thread = BorgDeleteThread(params['cmd'], params, parent=self)
+            thread = BorgDeleteThread(params['cmd'], params, parent=self.app)
             thread.updated.connect(self._set_status)
             thread.result.connect(self.delete_result)
             self._toggle_all_buttons(False)
@@ -429,20 +455,15 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
     def delete_result(self, result):
         if result['returncode'] == 0:
             self._set_status(self.tr('Archive deleted.'))
-            self.list_action()
+            deleted_row = self.archiveTable.findItems(self.archive_name, QtCore.Qt.MatchExactly)[0].row()
+            self.archiveTable.removeRow(deleted_row)
+            ArchiveModel.get(name=self.archive_name).delete_instance()
+            del self.archive_name
         else:
             self._toggle_all_buttons(True)
 
     def diff_action(self):
-        profile = self.profile()
-
-        window = DiffDialog(self.archiveTable)
-        self._toggle_all_buttons(True)
-        window.setParent(self, QtCore.Qt.Sheet)
-        self._window = window  # for testing
-        window.show()
-
-        if window.exec_():
+        def process_result():
             if window.selected_archives:
                 self.selected_archives = window.selected_archives
             archive_cell_newer = self.archiveTable.item(self.selected_archives[0], 4)
@@ -457,13 +478,22 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
 
                 if params['ok']:
                     self._toggle_all_buttons(False)
-                    thread = BorgDiffThread(params['cmd'], params, parent=self)
+                    thread = BorgDiffThread(params['cmd'], params, parent=self.app)
                     thread.updated.connect(self.mountErrors.setText)
                     thread.result.connect(self.list_diff_result)
                     thread.start()
                 else:
                     self._set_status(params['message'])
 
+        profile = self.profile()
+
+        window = DiffDialog(self.archiveTable)
+        self._toggle_all_buttons(True)
+        window.setParent(self, QtCore.Qt.Sheet)
+        self._window = window  # for testing
+        window.show()
+        window.accepted.connect(process_result)
+
     def list_diff_result(self, result):
         self._set_status('')
         if result['returncode'] == 0:
@@ -474,3 +504,48 @@ class ArchiveTab(ArchiveTabBase, ArchiveTabUI, BackupProfileMixin):
             window.setParent(self, QtCore.Qt.Sheet)
             self._resultwindow = window  # for testing
             window.show()
+
+    def rename_action(self):
+        profile = self.profile()
+        params = BorgRenameThread.prepare(profile)
+        if not params['ok']:
+            self._set_status(params['message'])
+            return
+
+        archive_name = self.selected_archive_name()
+        if archive_name is not None:
+            new_name, finished = QInputDialog.getText(
+                self,
+                self.tr("Change name"),
+                self.tr("New archive name:"),
+                text=archive_name)
+
+            if not finished:
+                return
+
+            if not new_name:
+                self._set_status(self.tr('Archive name cannot be blank.'))
+                return
+
+            new_name_exists = ArchiveModel.get_or_none(name=new_name, repo=profile.repo)
+            if new_name_exists is not None:
+                self._set_status(self.tr('An archive with this name already exists.'))
+                return
+
+            params['cmd'][-1] += f'::{archive_name}'
+            params['cmd'].append(new_name)
+
+            thread = BorgRenameThread(params['cmd'], params, parent=self)
+            thread.updated.connect(self._set_status)
+            thread.result.connect(self.rename_result)
+            self._toggle_all_buttons(False)
+            thread.start()
+        else:
+            self._set_status(self.tr("No archive selected"))
+
+    def rename_result(self, result):
+        if result['returncode'] == 0:
+            self._set_status(self.tr('Archive renamed.'))
+            self.populate_from_profile()
+        else:
+            self._toggle_all_buttons(True)
